package com.tourism.webadmin.app.website.service.impl;

import com.github.pagehelper.page.PageMethod;
import com.tourism.common.additional.model.FileUrlObject;
import com.tourism.common.additional.utils.JsonUtils;
import com.tourism.common.additional.utils.StringUtils;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.webadmin.app.website.dto.TourismProjectTravelNotesToWebDto;
import com.tourism.webadmin.app.website.service.BasicToWebService;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.BannerInfoService;
import com.tourism.webadmin.back.service.DirectoryInfoService;
import com.tourism.webadmin.back.service.TourTourismProjectTravelNotesService;
import com.tourism.webadmin.back.service.TourUserLikeTravelNotesService;
import com.tourism.webadmin.back.vo.BannerInfoVo;
import com.tourism.webadmin.back.vo.DirectoryInfoVo;
import com.tourism.webadmin.back.vo.TourTourismProjectTravelNotesVo;
import com.tourism.webadmin.config.ApplicationConfig;
import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 门户网站首页Service
 * 
 * @author tourism
 */
@Slf4j
@Service("BasicToWebService")
public class BasicToWebServiceImpl implements BasicToWebService
{

    @Autowired
    private BannerInfoService bannerInfoService;
    @Autowired
    private DirectoryInfoService directoryInfoService;
    @Autowired
    private ApplicationConfig applicationConfig;
    @Autowired
    private TourTourismProjectTravelNotesService tourTourismProjectTravelNotesService;
    @Autowired
    private TourUserLikeTravelNotesService tourUserLikeTravelNotesService;

    @Override
    public MyPageData<BannerInfoVo> getBannerInfoList(Long belongTab) {
        BannerInfo bannerInfoFilter = new BannerInfo();
        bannerInfoFilter.setBelongTab(belongTab);
        //首页展示的话，只能展示启用的banner图
        bannerInfoFilter.setEnable(1);
        MyOrderParam.OrderInfo orderInfo = new MyOrderParam.OrderInfo();
        orderInfo.setFieldName("mainBanner");
        orderInfo.setAsc(false);
        MyOrderParam myOrderParam  = new MyOrderParam();
        myOrderParam.add(orderInfo);
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, BannerInfo.class);
        List<BannerInfo> bannerInfoList = bannerInfoService.getBannerInfoListWithRelation(bannerInfoFilter, orderBy);
        MyPageData<BannerInfoVo> bannerInfoVoMyPageData = MyPageUtil.makeResponseData(bannerInfoList, BannerInfoVo.class);

        List<BannerInfoVo> dataList = bannerInfoVoMyPageData.getDataList();
        //先把imgUrl由jaon转换为List<FileUrlObject>
        dataList.stream().forEach(item ->
        {
            if (StringUtils.isNotEmpty(item.getImgUrl())) {
                List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(item.getImgUrl(), FileUrlObject.class);
                if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                    ArrayList<String> arrayList = new ArrayList<>();
                    //遍历每个元素的数组对象,对元素的url对象进行拼接
                    for (FileUrlObject y : fileUrlObjectList) {
                        String imgUrlAfterConvert =
                                applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + item.getId() + "&fieldName=imgUrl&asImage=true&filename=" + y.getFilename();
                        arrayList.add(imgUrlAfterConvert);
                    }
                    item.setImgUrlsAfterConvert(arrayList);
                }
            }
        });

        return bannerInfoVoMyPageData;
    }

    @Override
    public MyPageData<DirectoryInfoVo> getDirectoryInfoList(Long parentId, Integer isHotspot, Integer isAll) {

        DirectoryInfo directoryInfoFilter = new DirectoryInfo();
        //当directoryId为1000时,则表明查询三级菜单
        if(parentId != null){
            directoryInfoFilter.setParentId(parentId);
        }
        if(isAll!= null && isAll == 1){
            directoryInfoFilter.setDirectoryId(1000L);
            //菜单类型：1：旅游，2：劳务
            directoryInfoFilter.setDirectoryType(1);
        }
        if(isHotspot != null){
            directoryInfoFilter.setIsHotspot(isHotspot);
        }
        MyOrderParam myOrderParam = new MyOrderParam();
        MyOrderParam.OrderInfo orderInfo = new MyOrderParam.OrderInfo();
        //先根据是否热点排序，再根据展示顺序排序
        orderInfo.setFieldName("isHotspot");
        orderInfo.setAsc(false);
        myOrderParam.add(orderInfo);
        orderInfo.setFieldName("showOrder");
        orderInfo.setAsc(true);
        myOrderParam.add(orderInfo);
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, DirectoryInfo.class);
        //看到的项目为启用的项目
        directoryInfoFilter.setEnable(1);
        List<DirectoryInfo> directoryInfoList =
                directoryInfoService.getDirectoryInfoListWithRelation(directoryInfoFilter, orderBy);

        MyPageData<DirectoryInfoVo> directoryInfoVoMyPageData = MyPageUtil.makeResponseData(directoryInfoList, DirectoryInfoVo.class);

        List<DirectoryInfoVo> dataList = directoryInfoVoMyPageData.getDataList();
        //先把imgUrl由jaon转换为List<FileUrlObject>
        dataList.stream().forEach(item ->
        {
            if (StringUtils.isNotEmpty(item.getHotPictureUrl())) {
                List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(item.getHotPictureUrl(), FileUrlObject.class);
                if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                    ArrayList<String> arrayList = new ArrayList<>();
                    //遍历每个元素的数组对象,对元素的url对象进行拼接
                    for (FileUrlObject y : fileUrlObjectList) {
                        String hotPictureUrlsAfterConvert =
                                applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + item.getId() + "&fieldName=hotPictureUrl&asImage=true&filename=" + y.getFilename();
                        arrayList.add(hotPictureUrlsAfterConvert);
                    }
                    item.setHotPictureUrlsAfterConvert(arrayList);
                }
            }
        });

        return directoryInfoVoMyPageData;
    }

    @Override
    public MyPageData<TourTourismProjectTravelNotesVo> getTourTourismProjectTravelNotesList(TourismProjectTravelNotesToWebDto tourismProjectTravelNotesToWebDto){

        TourTourismProjectTravelNotes tourTourismProjectTravelNotesFilter = new TourTourismProjectTravelNotes();
        tourTourismProjectTravelNotesFilter.setEnable(1);
        tourTourismProjectTravelNotesFilter.setIsHotspot(1);
        if(tourismProjectTravelNotesToWebDto.getPageNum() != null && tourismProjectTravelNotesToWebDto.getPageSize() != null) {
            PageMethod.startPage(tourismProjectTravelNotesToWebDto.getPageNum(), tourismProjectTravelNotesToWebDto.getPageSize(), true);
        }
        MyOrderParam myOrderParam = new MyOrderParam();
        myOrderParam.add(new MyOrderParam.OrderInfo("showOrder",true,null));
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourTourismProjectTravelNotes.class);
        List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList = tourTourismProjectTravelNotesService.getTourTourismProjectTravelNotesList(tourTourismProjectTravelNotesFilter, orderBy);
        MyPageData<TourTourismProjectTravelNotesVo> tourTourismProjectTravelNotesVoMyPageData = MyPageUtil.makeResponseData(tourTourismProjectTravelNotesList, TourTourismProjectTravelNotesVo.class);
        List<TourTourismProjectTravelNotesVo> dataList = tourTourismProjectTravelNotesVoMyPageData.getDataList();
        dataList.stream().forEach(item->{
            if (StringUtils.isNotEmpty(item.getHomeHotPicture())) {
                List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(item.getHomeHotPicture(), FileUrlObject.class);
                if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                    ArrayList<String> arrayList = new ArrayList<>();
                    //遍历每个元素的数组对象,对元素的url对象进行拼接
                    for (FileUrlObject y : fileUrlObjectList) {
                        String imgUrlAfterConvert =
                                applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + item.getId() + "&fieldName=homeHotPicture&asImage=true&filename=" + y.getFilename();
                        arrayList.add(imgUrlAfterConvert);
                    }
                    item.setHomeHotPicturesAfterConvert(arrayList);
                }
            }
        });

        return tourTourismProjectTravelNotesVoMyPageData;
    }

    @Override
    public MyPageData<TourTourismProjectTravelNotesVo> getTravelNotesPageList(TourismProjectTravelNotesToWebDto tourismProjectTravelNotesToWebDto){
        //当belongTab为1的时候，则表明查询全部的游记
        MyOrderParam myOrderParam = new MyOrderParam();
        myOrderParam.add(new MyOrderParam.OrderInfo("isHotspot",false,null));
        myOrderParam.add(new MyOrderParam.OrderInfo("showOrder",true,null));
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourTourismProjectTravelNotes.class);

        TourTourismProjectTravelNotes tourTourismProjectTravelNotes = new TourTourismProjectTravelNotes();
        Long belongTabLong = Long.parseLong(tourismProjectTravelNotesToWebDto.getBelongTab());

        if(tourismProjectTravelNotesToWebDto.getAreaId() == null && tourismProjectTravelNotesToWebDto.getCountryId() == null){
            //判断是否为全部的地区
            tourTourismProjectTravelNotes.setEnable(1);
            tourTourismProjectTravelNotes.setBelongTab(null);
        }else if(tourismProjectTravelNotesToWebDto.getAreaId() != null && tourismProjectTravelNotesToWebDto.getCountryId() == null){
            //判断是否为该地区的所有国家
            DirectoryInfo directoryInfo = new DirectoryInfo();
            directoryInfo.setParentId(belongTabLong);
            directoryInfo.setEnable(1);
            List<DirectoryInfo> directoryInfoList =
                    directoryInfoService.getDirectoryInfoList(directoryInfo, "");
            List<Long> directoryIdList =
                    directoryInfoList.stream().map(DirectoryInfo::getId).collect(Collectors.toList());
            tourTourismProjectTravelNotes.setBelongTab(null);
            tourTourismProjectTravelNotes.setDirectoryInfoIds(directoryIdList);
        }else {
            //如果查询的是某个国家的游记的话
            tourTourismProjectTravelNotes.setBelongTab(belongTabLong);
        }

        if(tourismProjectTravelNotesToWebDto.getPageNum() != null && tourismProjectTravelNotesToWebDto.getPageSize() != null) {
            PageMethod.startPage(tourismProjectTravelNotesToWebDto.getPageNum(), tourismProjectTravelNotesToWebDto.getPageSize(), true);
        }

        List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList =
                tourTourismProjectTravelNotesService.getTourTourismProjectTravelNotesList(tourTourismProjectTravelNotes, orderBy);

        MyPageData<TourTourismProjectTravelNotesVo> tourTourismProjectTravelNotesVoMyPageData =
                MyPageUtil.makeResponseData(tourTourismProjectTravelNotesList, TourTourismProjectTravelNotesVo.class);

        List<TourTourismProjectTravelNotesVo> dataList = tourTourismProjectTravelNotesVoMyPageData.getDataList();

        dataList.stream().forEach(item->{
            if(StringUtils.isNotEmpty(item.getTourismUrl())){
                List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(item.getTourismUrl(), FileUrlObject.class);
                if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                    ArrayList<String> arrayList = new ArrayList<>();
                    //遍历每个元素的数组对象,对元素的url对象进行拼接
                    for (FileUrlObject y : fileUrlObjectList) {
                        String imgUrlAfterConvert =
                                applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + item.getId() + "&fieldName=tourismUrl&asImage=true&filename=" + y.getFilename();
                        arrayList.add(imgUrlAfterConvert);
                    }
                    item.setTourismUrlsAfterConvert(arrayList);
                }
            }
        });

        return tourTourismProjectTravelNotesVoMyPageData;
    }

    @Override
    public TourTourismProjectTravelNotesVo travelNotesDetail(Long id){
        TourTourismProjectTravelNotes tourTourismProjectTravelNotes =
                tourTourismProjectTravelNotesService.getByIdWithRelation(id, MyRelationParam.full());

        TourTourismProjectTravelNotesVo tourTourismProjectTravelNotesVo =
                MyModelUtil.copyTo(tourTourismProjectTravelNotes, TourTourismProjectTravelNotesVo.class);

        if(StringUtils.isNotEmpty(tourTourismProjectTravelNotesVo.getTourismUrl())){
            List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(tourTourismProjectTravelNotesVo.getTourismUrl(), FileUrlObject.class);
            if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                ArrayList<String> arrayList = new ArrayList<>();
                //遍历每个元素的数组对象,对元素的url对象进行拼接
                for (FileUrlObject y : fileUrlObjectList) {
                    String imgUrlAfterConvert =
                            applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + tourTourismProjectTravelNotesVo.getId() + "&fieldName=tourismUrl&asImage=true&filename=" + y.getFilename();
                    arrayList.add(imgUrlAfterConvert);
                }
                tourTourismProjectTravelNotesVo.setTourismUrlsAfterConvert(arrayList);
            }
        }

        if (StringUtils.isNotEmpty(tourTourismProjectTravelNotesVo.getHomeHotPicture())) {
            List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(tourTourismProjectTravelNotesVo.getHomeHotPicture(), FileUrlObject.class);
            if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                ArrayList<String> arrayList = new ArrayList<>();
                //遍历每个元素的数组对象,对元素的url对象进行拼接
                for (FileUrlObject y : fileUrlObjectList) {
                    String hotPictureUrlsAfterConvert =
                            applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + tourTourismProjectTravelNotesVo.getId() + "&fieldName=homeHotPicturesAfterConvert&asImage=true&filename=" + y.getFilename();
                    arrayList.add(hotPictureUrlsAfterConvert);
                }
                tourTourismProjectTravelNotesVo.setHomeHotPicturesAfterConvert(arrayList);
            }
        }

        if (StringUtils.isNotEmpty(tourTourismProjectTravelNotesVo.getTravelNotesBanner())) {
            List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(tourTourismProjectTravelNotesVo.getTravelNotesBanner(), FileUrlObject.class);
            if (!CollectionUtils.isEmpty(fileUrlObjectList)) {
                ArrayList<String> arrayList = new ArrayList<>();
                //遍历每个元素的数组对象,对元素的url对象进行拼接
                for (FileUrlObject y : fileUrlObjectList) {
                    String hotPictureUrlsAfterConvert =
                            applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + tourTourismProjectTravelNotesVo.getId() + "&fieldName=travelNotesBanner&asImage=true&filename=" + y.getFilename();
                    arrayList.add(hotPictureUrlsAfterConvert);
                }
                tourTourismProjectTravelNotesVo.setTravelNotesBannerAfterConvert(arrayList);
            }
        }

        return tourTourismProjectTravelNotesVo;
    }

    @Override
    public void userLikeTravelNotesUpdate(Long travelNotesId,Integer type){

        TourUserLikeTravelNotes tourUserLikeTravelNotes =
                new TourUserLikeTravelNotes();

        tourUserLikeTravelNotes.setTravelNotesId(travelNotesId);
        tourUserLikeTravelNotes.setUserId(TokenData.takeFromRequest().getUserId());

        if(type == 0){
            tourUserLikeTravelNotesService.removeBy(tourUserLikeTravelNotes);
        }else {
            tourUserLikeTravelNotesService.saveNew(tourUserLikeTravelNotes);
        }
    }

    @Override
    public Boolean isLikeTravelNotes(Long travelNotesId){

        TourUserLikeTravelNotes tourUserLikeTravelNotes = new TourUserLikeTravelNotes();

        tourUserLikeTravelNotes.setTravelNotesId(travelNotesId);
        tourUserLikeTravelNotes.setUserId(TokenData.takeFromRequest().getUserId());
        return tourUserLikeTravelNotesService.existByFilter(tourUserLikeTravelNotes);

    }

    @Override
    public void travelNotesViewCountAdd(Long travelNotesId){

        TourTourismProjectTravelNotes tourTourismProjectTravelNotes = tourTourismProjectTravelNotesService.getById(travelNotesId);
        tourTourismProjectTravelNotes.setPageViewCount(tourTourismProjectTravelNotes.getPageViewCount()+1);

    }

}

