package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 用户游记点赞表操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "用户游记点赞表管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourUserLikeTravelNotes")
public class TourUserLikeTravelNotesController {

    @Autowired
    private TourUserLikeTravelNotesService tourUserLikeTravelNotesService;

//    /**
//     * 新增用户游记点赞表数据。
//     *
//     * @param tourUserLikeTravelNotesDto 新增对象。
//     * @return 应答结果对象，包含新增对象主键Id。
//     */
//    @ApiOperationSupport(ignoreParameters = {"tourUserLikeTravelNotesDto.id"})
////    @SaCheckPermission("tourUserLikeTravelNotes.add")
//    @OperationLog(type = SysOperationLogType.ADD)
//    @PostMapping("/add")
//    public ResponseResult<Long> add(@MyRequestBody TourUserLikeTravelNotesDto tourUserLikeTravelNotesDto) {
//        String errorMessage = MyCommonUtil.getModelValidationError(tourUserLikeTravelNotesDto, false);
//        if (errorMessage != null) {
//            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
//        }
//        TourUserLikeTravelNotes tourUserLikeTravelNotes = MyModelUtil.copyTo(tourUserLikeTravelNotesDto, TourUserLikeTravelNotes.class);
//        tourUserLikeTravelNotes = tourUserLikeTravelNotesService.saveNew(tourUserLikeTravelNotes);
//        return ResponseResult.success(tourUserLikeTravelNotes.getId());
//    }
//
//    /**
//     * 更新用户游记点赞表数据。
//     *
//     * @param tourUserLikeTravelNotesDto 更新对象。
//     * @return 应答结果对象。
//     */
//    @SaCheckPermission("tourUserLikeTravelNotes.update")
//    @OperationLog(type = SysOperationLogType.UPDATE)
//    @PostMapping("/update")
//    public ResponseResult<Void> update(@MyRequestBody TourUserLikeTravelNotesDto tourUserLikeTravelNotesDto) {
//        String errorMessage = MyCommonUtil.getModelValidationError(tourUserLikeTravelNotesDto, true);
//        if (errorMessage != null) {
//            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
//        }
//        TourUserLikeTravelNotes tourUserLikeTravelNotes = MyModelUtil.copyTo(tourUserLikeTravelNotesDto, TourUserLikeTravelNotes.class);
//        TourUserLikeTravelNotes originalTourUserLikeTravelNotes = tourUserLikeTravelNotesService.getById(tourUserLikeTravelNotes.getId());
//        if (originalTourUserLikeTravelNotes == null) {
//            // NOTE: 修改下面方括号中的话述
//            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
//            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
//        }
//        if (!tourUserLikeTravelNotesService.update(tourUserLikeTravelNotes, originalTourUserLikeTravelNotes)) {
//            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
//        }
//        return ResponseResult.success();
//    }

    /**
     * 删除用户游记点赞表数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourUserLikeTravelNotes.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除用户游记点赞表数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourUserLikeTravelNotes.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的用户游记点赞表列表。
     *
     * @param tourUserLikeTravelNotesDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourUserLikeTravelNotes.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourUserLikeTravelNotesVo>> list(
            @MyRequestBody TourUserLikeTravelNotesDto tourUserLikeTravelNotesDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourUserLikeTravelNotes tourUserLikeTravelNotesFilter = MyModelUtil.copyTo(tourUserLikeTravelNotesDtoFilter, TourUserLikeTravelNotes.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourUserLikeTravelNotes.class);
        List<TourUserLikeTravelNotes> tourUserLikeTravelNotesList =
                tourUserLikeTravelNotesService.getTourUserLikeTravelNotesListWithRelation(tourUserLikeTravelNotesFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourUserLikeTravelNotesList, TourUserLikeTravelNotesVo.class));
    }

    /**
     * 查看指定用户游记点赞表对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourUserLikeTravelNotes.view")
    @GetMapping("/view")
    public ResponseResult<TourUserLikeTravelNotesVo> view(@RequestParam Long id) {
        TourUserLikeTravelNotes tourUserLikeTravelNotes = tourUserLikeTravelNotesService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourUserLikeTravelNotes == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourUserLikeTravelNotesVo tourUserLikeTravelNotesVo = MyModelUtil.copyTo(tourUserLikeTravelNotes, TourUserLikeTravelNotesVo.class);
        return ResponseResult.success(tourUserLikeTravelNotesVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourUserLikeTravelNotes originalTourUserLikeTravelNotes = tourUserLikeTravelNotesService.getById(id);
        if (originalTourUserLikeTravelNotes == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourUserLikeTravelNotesService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
