package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.CompanyType;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.Date;

/**
 * 公司信息管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "CompanyInfoDto对象")
@Data
public class CompanyInfoDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 公司名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "公司名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，公司名称不能为空！")
    private String companyName;

    /**
     * 公司地址。
     */
    @Schema(description = "公司地址。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，公司地址不能为空！")
    private String address;

    /**
     * 成立日期。
     */
    @Schema(description = "成立日期。")
    private Date establishDate;

    /**
     * 联系电话。
     */
    @Schema(description = "联系电话。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，联系电话不能为空！")
    private String telephone;

    /**
     * 公司网站链接。
     */
    @Schema(description = "公司网站链接。")
    private String website;

    /**
     * 公司类型。
     */
    @Schema(description = "公司类型。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，公司类型不能为空！")
    @ConstDictRef(constDictClass = CompanyType.class, message = "数据验证失败，公司类型为无效值！")
    private Integer companyType;

    /**
     * 公司邮箱。
     */
    @Schema(description = "公司邮箱。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，公司邮箱不能为空！")
    private String email;

    /**
     * 公司图片上传地址。
     */
    @Schema(description = "公司图片上传地址。")
    private String companyUrl;

    /**
     * 更新用户。
     */
    @Schema(description = "更新用户。")
    private Long updateUserId;

    /**
     * company_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
