package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;

/**
 * 外卖订单明细管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "DeliveryOrderItemsDto对象")
@Data
public class DeliveryOrderItemsDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 食品名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "食品名称。可支持等于操作符的列表数据过滤。")
    private String foodName;

    /**
     * 订单id。
     */
    @Schema(description = "订单id。")
    private Long orderId;

    /**
     * 价格。
     */
    @Schema(description = "价格。")
    private BigDecimal price;

    /**
     * 数量。
     */
    @Schema(description = "数量。")
    private Integer number;

    /**
     * 删除标记(1: 正常 -1: 已删除)。
     */
    @Schema(description = "删除标记(1: 正常 -1: 已删除)。")
    private Integer dataState;
}
