package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;
import com.tourism.webadmin.back.model.constant.CompanyType;
import com.tourism.webadmin.back.model.constant.Hotspot;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 门户菜单管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "DirectoryInfoDto对象")
@Data
public class DirectoryInfoDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 父级id。
     */
    @Schema(description = "父级id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，父级id不能为空！")
    private Long parentId;

    /**
     * 菜单名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "菜单名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，菜单名称不能为空！")
    private String menuName;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，显示顺序不能为空！")
    private Integer showOrder;

    /**
     * 是否启用。
     */
    @Schema(description = "是否启用，0禁用，1启用。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，是否启用不能为空！")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用为无效值！")
    private Integer enable;

    /**
     * 菜单类型。
     */
    @Schema(description = "菜单类型：1：旅游，2：劳务。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，菜单类型不能为空！")
    @ConstDictRef(constDictClass = CompanyType.class, message = "数据验证失败，菜单类型为无效值！")
    private Integer directoryType;

    /**
     * 是否热点，0非热点，1热点。
     */
    @Schema(description = "是否热点，0非热点，1热点。")
    @ConstDictRef(constDictClass = Hotspot.class, message = "数据验证失败，是否热点，0非热点，1热点为无效值！")
    private Integer isHotspot;

    /**
     * 热门排序。
     */
    @Schema(description = "热门排序。")
    private Integer hotOrder;

    /**
     * 劳务图片。
     */
    @Schema(description = "热点图片。")
    private String hotPictureUrl;

    /**
     * menu_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
