package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Hotspot;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * 劳务项目管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "JobProjectDto对象")
@Data
public class JobProjectDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 出国劳务标题。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "出国劳务标题。可支持等于操作符的列表数据过滤。")
    private String jobTitle;

    /**
     * 标签。
     */
    @Schema(description = "标签。")
    private String jobLabel;

    /**
     * 所属分类。
     */
    @Schema(description = "所属分类。")
    private Long belongTab;

    /**
     * 工作国家或地区。
     */
    @Schema(description = "工作国家或地区。")
    private String jobArea;

    /**
     * 招聘人数。
     */
    @Schema(description = "招聘人数。")
    private Integer jobPeople;

    /**
     * 学历要求。
     */
    @Schema(description = "学历要求。")
    private String jobEducation;

    /**
     * 语言要求。
     */
    @Schema(description = "语言要求。")
    private String jobLanguage;

    /**
     * 合同期限。
     */
    @Schema(description = "合同期限。")
    private String jobContract;

    /**
     * 截至日期。
     */
    @Schema(description = "截至日期。")
    private String jobDeadline;

    /**
     * 岗位要求。
     */
    @Schema(description = "岗位要求。")
    private String jobRequire;

    /**
     * 福利待遇。
     */
    @Schema(description = "福利待遇。")
    private String jobWelfare;

    /**
     * 工作时间。
     */
    @Schema(description = "工作时间。")
    private String jobTime;

    /**
     * 招聘岗位。
     */
    @Schema(description = "招聘岗位。")
    private String jobPost;

    /**
     * 项目优势。
     */
    @Schema(description = "项目优势。")
    private String jobAdvantage;

    /**
     * 劳务图片。
     */
    @Schema(description = "劳务图片。")
    private String jobUrl;

    /**
     * 是否热点。
     */
    @Schema(description = "是否热点，0非热点，1热点。")
    @ConstDictRef(constDictClass = Hotspot.class, message = "数据验证失败，是否热点为无效值！")
    private Integer isHotspot;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。")
    private Integer showOrder;

    /**
     * 是否启用。
     */
    @Schema(description = "是否启用，0禁用，1启用。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用为无效值！")
    private Integer enable;

    /**
     * 出国劳务简述。
     */
    @Schema(description = "出国劳务简述。")
    private String remarks;

    /**
     * 工资上限。
     */
    @Schema(description = "工资上限。")
    private BigDecimal salaryCap;

    /**
     * 工资下限。
     */
    @Schema(description = "工资下限。")
    private BigDecimal salaryFloor;

    /**
     * 工资单位。
     */
    @Schema(description = "工资单位。")
    private String salaryUnit;

    /**
     * 城市(二级)。
     */
    @Schema(description = "城市(二级)。")
    private String city;

    /**
     * 详细地址。
     */
    @Schema(description = "详细地址。")
    private String address;

    /**
     * 浏览量。
     */
    @Schema(description = "浏览量。")
    private Integer pageViewCount;

    /**
     * 招聘部门。
     */
    @Schema(description = "招聘部门。")
    private String dept;

    /**
     * 性别要求(1:男 2:女 3:不限)。
     */
    @Schema(description = "性别要求(1:男 2:女 3:其它)。")
    private String sex;

    /**
     * 年龄上限。
     */
    @Schema(description = "年龄上限。")
    private Integer ageCap;

    /**
     * 年龄下限。
     */
    @Schema(description = "年龄下限。")
    private Integer ageFloor;

    /**
     * 收藏量。
     */
    @Schema(description = "收藏量。")
    private Integer favouriteCount;

    /**
     * job_title LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 是否急聘(0:否 1:是)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否急聘(0:否 1:是)。可支持等于操作符的列表数据过滤。")
    private Integer isUrgent;

    /**
     * 菜单id。
     */
    private List<Long> directoryInfoIds;
}
