package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * 餐馆美食信息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "RestaurantFoodInfoDto对象")
@Data
public class RestaurantFoodInfoDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 饭馆美食种类id。
     */
    @Schema(description = "饭馆美食种类id。")
    private Long foodTypeId;

    /**
     * 菜品名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "菜品名称。可支持等于操作符的列表数据过滤。")
    private String name;

    /**
     * 菜品图片。
     */
    @Schema(description = "菜品图片。")
    private String url;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0禁用，1启用为无效值！")
    private Integer enable;

    /**
     * 菜品销量。
     */
    @Schema(description = "菜品销量。")
    private Integer sales;

    /**
     * 菜品价格。
     */
    @Schema(description = "菜品价格。")
    private BigDecimal price;

    /**
     * 价格单位。
     */
    @Schema(description = "价格单位。")
    private String priceUnit;

    /**
     * 用户所在部门Id。
     */
    @Schema(description = "用户所在部门Id。")
    private Long restaurantId;

    /**
     * 打包费。
     */
    @Schema(description = "打包费。")
    private BigDecimal balingCharge;

    /**
     * name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 餐馆idList。
     */
    @Schema(description = "餐馆idList")
    private List<Long> restaurantIdList;
}
