package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.List;

/**
 * 餐馆美食类型Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "RestaurantFoodTypeDto对象")
@Data
public class RestaurantFoodTypeDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 种类名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "种类名称。可支持等于操作符的列表数据过滤。")
    private String name;

    /**
     * 餐馆id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "餐馆id。可支持等于操作符的列表数据过滤。")
    private Long restaurantId;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。")
    private Integer enable;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。")
    private Integer showOrder;

    /**
     * 描述。
     */
    @Schema(description = "描述。")
    private String description;

    /**
     * name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 餐馆idList。
     */
    @Schema(description = "餐馆idList")
    private List<Long> restaurantIdList;
}
