package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * 餐馆信息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "RestaurantInfoDto对象")
@Data
public class RestaurantInfoDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 店铺名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "店铺名称。可支持等于操作符的列表数据过滤。")
    private String name;

    /**
     * 店铺标签。
     */
    @Schema(description = "店铺标签。")
    private String tag;

    /**
     * 店铺种类。
     */
    @Schema(description = "店铺种类。")
    private String typeId;

    /**
     * 店铺公告。
     */
    @Schema(description = "店铺公告。")
    private String announcement;

    /**
     * 工作时间段(例:8:00-19:00)。
     */
    @Schema(description = "工作时间段(例:8:00-19:00)。")
    private String workTime;

    /**
     * 店铺地址。
     */
    @Schema(description = "店铺地址。")
    private String address;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0禁用，1启用为无效值！")
    private Integer enable;

    /**
     * 店铺图片。
     */
    @Schema(description = "店铺图片。")
    private String url;

    /**
     * 店铺环境图片。
     */
    @Schema(description = "店铺环境图片。")
    private String environmentImage;

    /**
     * 联系方式。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系方式。可支持等于操作符的列表数据过滤。")
    private String contactInformation;

    /**
     * 经度。
     */
    @Schema(description = "经度。")
    private BigDecimal longitude;

    /**
     * 纬度。
     */
    @Schema(description = "纬度。")
    private BigDecimal latitude;

    /**
     * 销量。
     */
    @Schema(description = "销量。")
    private Long totalSales;

    /**
     * 所处国家。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "所处国家。可支持等于操作符的列表数据过滤。")
    private String contry;

    /**
     * 所处城市。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "所处城市。可支持等于操作符的列表数据过滤。")
    private String city;

    /**
     * 配送费。
     */
    @Schema(description = "配送费。")
    private String deliveryCharge;

    /**
     * 最低消费。
     */
    @Schema(description = "最低消费。")
    private BigDecimal minCharge;

    /**
     * 用户所在部门Id。
     */
    @Schema(description = "用户所在部门Id。")
    private Long deptId;

    /**
     * name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 餐馆idList。
     */
    @Schema(description = "餐馆idList")
    private List<Long> restaurantIdList;
}
