package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.Date;

/**
 * 门户预定管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "TourBookInfoDto对象")
@Data
public class TourBookInfoDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 预定电话。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "预定电话。可支持等于操作符的列表数据过滤。")
    private String bookMobile;

    /**
     * 预定姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "预定姓名。可支持等于操作符的列表数据过滤。")
    private String bookName;

    /**
     * 预定的ip。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "预定的ip。可支持等于操作符的列表数据过滤。")
    private String bookIp;

    /**
     * 预定时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "预定时间。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，预定时间不能为空！")
    private Date bookTime;

    /**
     * 是否处理(0:未处理 1:已处理)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否处理(0:未处理 1:已处理)。可支持等于操作符的列表数据过滤。")
    private String isHandle;

    /**
     * 预定的种类(1:旅游 2:劳务)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "预定的种类(1:旅游 2:劳务)。可支持等于操作符的列表数据过滤。")
    private Integer type;

    /**
     * 预定种类的id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "预定种类的id。可支持等于操作符的列表数据过滤。")
    private Long projectId;

    /**
     * book_mobile / book_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    @Schema(description = "开始时间")
    private Date startTime;

    @Schema(description = "结束时间")
    private Date endTime;
}
