package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Hotspot;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 旅游项目游记管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "TourTourismProjectTravelNotesDto对象")
@Data
public class TourTourismProjectTravelNotesDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 项目标题。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "项目标题。可支持等于操作符的列表数据过滤。")
    private String projectTitle;

    /**
     * 项目简述。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "项目简述。可支持等于操作符的列表数据过滤。")
    private String remarks;

    /**
     * 是否热点，0非热点，1热点。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否热点，0非热点，1热点。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Hotspot.class, message = "数据验证失败，是否热点，0非热点，1热点为无效值！")
    private Integer isHotspot;

    /**
     * 标签。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "标签。可支持等于操作符的列表数据过滤。")
    private String projectLabel;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。")
    private Integer showOrder;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0禁用，1启用为无效值！")
    private Integer enable;

    /**
     * 所属分类。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "所属分类。可支持等于操作符的列表数据过滤。")
    private Long belongTab;

    /**
     * 出发地。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "出发地。可支持等于操作符的列表数据过滤。")
    private String startPlace;

    /**
     * 目的地。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "目的地。可支持等于操作符的列表数据过滤。")
    private String endPlace;

    /**
     * 总天数。
     */
    @Schema(description = "总天数。")
    private String countTimes;

    /**
     * 项目价格。
     */
    @Schema(description = "项目价格。")
    private BigDecimal price;

    /**
     * 项目展示图片。
     */
    @Schema(description = "项目展示图片。")
    private String tourismUrl;

    /**
     * 价格单位。
     */
    @Schema(description = "价格单位。")
    private String priceUnit;

    /**
     * 服务保障。
     */
    @Schema(description = "服务保障。")
    private String serviceEnsure;

    /**
     * 联系电话。
     */
    @Schema(description = "联系电话。")
    private String contactNumber;

    /**
     * 产品卖点。
     */
    @Schema(description = "产品卖点。")
    private String sellingPoint;

    /**
     * 短标题。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "短标题。可支持等于操作符的列表数据过滤。")
    private String shortTitle;

    /**
     * 短描述。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "短描述。可支持等于操作符的列表数据过滤。")
    private String shortDescription;

    /**
     * 首页热门图片。
     */
    @Schema(description = "首页热门图片。")
    private String homeHotPicture;

    /**
     * 联系人描述。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系人描述。可支持等于操作符的列表数据过滤。")
    private String contactDescription;

    /**
     * 游记banner。
     */
    @Schema(description = "游记banner。")
    private String travelNotesBanner;

    /**
     * 点赞量。
     */
    @Schema(description = "点赞量。")
    private Integer likeCount;

    /**
     * 浏览量。
     */
    @Schema(description = "浏览量。")
    private Integer pageViewCount;

    /**
     * 人物。
     */
    @Schema(description = "人物。")
    private String role;

    /**
     * 出发时间。
     */
    @Schema(description = "出发时间。")
    private Date departureTime;

    /**
     * 人均费用。
     */
    @Schema(description = "人均费用。")
    private String averageCost;

    /**
     * 推荐指数。
     */
    @Schema(description = "推荐指数。")
    private Integer recommendationRate;

    /**
     * remarks / project_label / start_place / end_place / short_title / short_description / contact_description LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 旅游项目(以逗号分割，最多为三个项目)。
     */
    @Schema(description = "旅游项目(以逗号分割，最多为三个项目)。")
    private String projectIds;
}
