package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.*;
import com.tourism.webadmin.back.model.constant.CompanyType;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.annotation.*;
import lombok.Data;

import java.util.Date;
import java.util.Map;

/**
 * 公司信息管理实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
@TableName(value = "tour_company_info")
public class CompanyInfo {

    /**
     * 主键id。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * 公司名称。
     */
    @TableField(value = "company_name")
    private String companyName;

    /**
     * 公司地址。
     */
    @TableField(value = "address")
    private String address;

    /**
     * 成立日期。
     */
    @TableField(value = "establish_date")
    private Date establishDate;

    /**
     * 联系电话。
     */
    @TableField(value = "telephone")
    private String telephone;

    /**
     * 公司网站链接。
     */
    @TableField(value = "website")
    private String website;

    /**
     * 公司类型。
     */
    @TableField(value = "company_type")
    private Integer companyType;

    /**
     * 公司邮箱。
     */
    @TableField(value = "email")
    private String email;

    /**
     * 公司图片上传地址。
     */
    @UploadFlagColumn(storeType = UploadStoreTypeEnum.HUAWEI_OBS_SYSTEM)
    @TableField(value = "company_url")
    private String companyUrl;

    /**
     * 创建用户。
     */
    @TableField(value = "create_user_id")
    private Long createUserId;

    /**
     * 创建时间。
     */
    @TableField(value = "create_time")
    private Date createTime;

    /**
     * 更新用户。
     */
    @TableField(value = "update_user_Id")
    private Long updateUserId;

    /**
     * 更新时间。
     */
    @TableField(value = "update_time")
    private Date updateTime;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * company_name LIKE搜索字符串。
     */
    @TableField(exist = false)
    private String searchString;

    public void setSearchString(String searchString) {
        this.searchString = MyCommonUtil.replaceSqlWildcard(searchString);
    }

    @RelationOneToOne(
            masterIdField = "id",
            slaveModelClass = CompanyContent.class,
            slaveIdField = "associationId")
    @TableField(exist = false)
    private CompanyContent companyContent;

    @RelationConstDict(
            masterIdField = "companyType",
            constantDictClass = CompanyType.class)
    @TableField(exist = false)
    private Map<String, Object> companyTypeDictMap;
}
