package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.tourism.common.core.annotation.MaskField;
import com.tourism.common.core.base.model.BaseModel;
import com.tourism.common.core.constant.MaskFieldTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 外卖订单管理实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_delivery_order")
public class DeliveryOrder extends BaseModel {

    /**
     * 主键。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * 配送时间。
     */
    @TableField(value = "delivery_time")
    private Date deliveryTime;

    /**
     * 配送地址。
     */
    @TableField(value = "delivery_address")
    private String deliveryAddress;

    /**
     * 收货电话。
     */
    @MaskField(maskType = MaskFieldTypeEnum.MOBILE_PHONE)
    @TableField(value = "delivery_phone")
    private String deliveryPhone;

    /**
     * 备注。
     */
    @TableField(value = "delivery_remark")
    private String deliveryRemark;

    /**
     * 餐具数量。
     */
    @TableField(value = "tableware")
    private Integer tableware;

    /**
     * 订单号。
     */
    @TableField(value = "order_no")
    private String orderNo;

    /**
     * 下单时间。
     */
    @TableField(value = "order_time")
    private Date orderTime;

    /**
     * 支付方式。
     */
    @TableField(value = "pay_type")
    private Integer payType;

    /**
     * 订单金额。
     */
    @TableField(value = "order_amount")
    private BigDecimal orderAmount;

    /**
     * 实付金额。
     */
    @TableField(value = "pay_amount")
    private BigDecimal payAmount;

    /**
     * 订单状态。
     */
    @TableField(value = "order_status")
    private Integer orderStatus;

    /**
     * 用户id。
     */
    @TableField(value = "customer_id")
    private Long customerId;

    /**
     * 退款状态。
     */
    @TableField(value = "refund_status")
    private Integer refundStatus;

    /**
     * 打包费。
     */
    @TableField(value = "pack_charge")
    private BigDecimal packCharge;

    /**
     * 配送费。
     */
    @TableField(value = "delivery_fee")
    private BigDecimal deliveryFee;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * 店铺名称。
     */
    @TableField(value = "shop_name")
    private String shopName;

    /**
     * 订单详情
     */
    @TableField(exist = false)
    private List<DeliveryOrderItems> itemList;
}
