package com.tourism.webadmin.back.service;

import com.alibaba.fastjson.JSONObject;
import com.tourism.common.core.base.service.IBaseService;
import com.tourism.webadmin.back.model.DeliveryOrder;

import java.util.*;

/**
 * 外卖订单管理数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface DeliveryOrderService extends IBaseService<DeliveryOrder, Long> {

    /**
     * 保存新增对象。
     *
     * @param deliveryOrder 新增对象。
     * @return 返回新增对象。
     */
    DeliveryOrder saveNew(DeliveryOrder deliveryOrder);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param deliveryOrderList 新增对象列表。
     */
    void saveNewBatch(List<DeliveryOrder> deliveryOrderList);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。通常适用于更大的插入数据量，如批量导入。
     *
     * @param deliveryOrderList 新增对象列表。
     * @param batchSize  每批插入的数量。如果该值小于等于0，则使用缺省值10000。
     */
    void saveNewBatch(List<DeliveryOrder> deliveryOrderList, int batchSize);

    /**
     * 保存新增主表对象及关联对象。
     *
     * @param deliveryOrder 新增主表对象。
     * @param relationData 全部关联从表数据。
     * @return 返回新增主表对象。
     */
    DeliveryOrder saveNewWithRelation(DeliveryOrder deliveryOrder, JSONObject relationData);

    /**
     * 更新数据对象。
     *
     * @param deliveryOrder         更新的对象。
     * @param originalDeliveryOrder 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(DeliveryOrder deliveryOrder, DeliveryOrder originalDeliveryOrder);

    /**
     * 更新主表对象及关联对象。
     *
     * @param deliveryOrder 主表对象新数据。
     * @param originalDeliveryOrder 主表对象源数据。
     * @param relationData 全部关联从表数据。
     * @return 修改成功返回true，否则false。
     */
    boolean updateWithRelation(DeliveryOrder deliveryOrder, DeliveryOrder originalDeliveryOrder, JSONObject relationData);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getDeliveryOrderListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<DeliveryOrder> getDeliveryOrderList(DeliveryOrder filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getDeliveryOrderList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<DeliveryOrder> getDeliveryOrderListWithRelation(DeliveryOrder filter, String orderBy);
}
