package com.tourism.webadmin.back.service;

import com.alibaba.fastjson.JSONObject;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.IBaseService;

import java.util.*;

/**
 * 劳务项目管理数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface JobProjectService extends IBaseService<JobProject, Long> {

    /**
     * 保存新增对象。
     *
     * @param jobProject 新增对象。
     * @return 返回新增对象。
     */
    JobProject saveNew(JobProject jobProject);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param jobProjectList 新增对象列表。
     */
    void saveNewBatch(List<JobProject> jobProjectList);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。通常适用于更大的插入数据量，如批量导入。
     *
     * @param jobProjectList 新增对象列表。
     * @param batchSize  每批插入的数量。如果该值小于等于0，则使用缺省值10000。
     */
    void saveNewBatch(List<JobProject> jobProjectList, int batchSize);

    /**
     * 保存新增主表对象及关联对象。
     *
     * @param jobProject 新增主表对象。
     * @param relationData 全部关联从表数据。
     * @return 返回新增主表对象。
     */
    JobProject saveNewWithRelation(JobProject jobProject, JSONObject relationData);

    /**
     * 更新数据对象。
     *
     * @param jobProject         更新的对象。
     * @param originalJobProject 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(JobProject jobProject, JobProject originalJobProject);

    /**
     * 更新主表对象及关联对象。
     *
     * @param jobProject 主表对象新数据。
     * @param originalJobProject 主表对象源数据。
     * @param relationData 全部关联从表数据。
     * @return 修改成功返回true，否则false。
     */
    boolean updateWithRelation(JobProject jobProject, JobProject originalJobProject, JSONObject relationData);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getJobProjectListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<JobProject> getJobProjectList(JobProject filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getJobProjectList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<JobProject> getJobProjectListWithRelation(JobProject filter, String orderBy);

    /**
     * 对批量导入数据列表进行数据合法性验证。
     * 验证逻辑主要覆盖主表的常量字典字段、字典表字典字段、数据源字段和一对一关联数据是否存在。
     *
     * @param dataList 主表的数据列表。
     * @param ignoreFieldSet 需要忽略校验的字典字段集合。通常对于字典反向翻译过来的字段适用，
     *                       避免了二次验证，以提升效率。
     * @return 验证结果。如果失败，包含具体的错误信息和导致错误的数据对象。
     */
    CallResult verifyImportList(List<JobProject> dataList, Set<String> ignoreFieldSet);
}
