package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 公司富文本管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("contentCompanyService")
public class ContentCompanyServiceImpl extends BaseService<ContentCompany, Long> implements ContentCompanyService {

    @Autowired
    private ContentCompanyMapper contentCompanyMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<ContentCompany> mapper() {
        return contentCompanyMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ContentCompany saveNew(ContentCompany contentCompany) {
        contentCompanyMapper.insert(this.buildDefaultValue(contentCompany));
        return contentCompany;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<ContentCompany> contentCompanyList) {
        if (CollUtil.isNotEmpty(contentCompanyList)) {
            contentCompanyList.forEach(this::buildDefaultValue);
            contentCompanyMapper.insertList(contentCompanyList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(ContentCompany contentCompany, ContentCompany originalContentCompany) {
        MyModelUtil.fillCommonsForUpdate(contentCompany, originalContentCompany);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<ContentCompany> uw = this.createUpdateQueryForNullValue(contentCompany, contentCompany.getId());
        return contentCompanyMapper.update(contentCompany, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return contentCompanyMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByAssociationId(Long associationId) {
        ContentCompany deletedObject = new ContentCompany();
        deletedObject.setAssociationId(associationId);
        return contentCompanyMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Override
    public List<ContentCompany> getContentCompanyList(ContentCompany filter, String orderBy) {
        return contentCompanyMapper.getContentCompanyList(filter, orderBy);
    }

    @Override
    public List<ContentCompany> getContentCompanyListWithRelation(ContentCompany filter, String orderBy) {
        List<ContentCompany> resultList = contentCompanyMapper.getContentCompanyList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private ContentCompany buildDefaultValue(ContentCompany contentCompany) {
        MyModelUtil.fillCommonsForInsert(contentCompany);
        contentCompany.setDataState(GlobalDeletedFlag.NORMAL);
        return contentCompany;
    }
}
