package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.Page;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.webadmin.back.dao.DeliveryOrderItemsMapper;
import com.tourism.webadmin.back.model.DeliveryOrderItems;
import com.tourism.webadmin.back.service.DeliveryOrderItemsService;
import com.tourism.webadmin.back.service.DeliveryOrderService;
import com.tourism.webadmin.back.vo.DeliveryOrderItemsVo;
import com.tourism.webadmin.config.DataSourceType;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * 外卖订单明细管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("deliveryOrderItemsService")
public class DeliveryOrderItemsServiceImpl extends BaseService<DeliveryOrderItems, Long> implements DeliveryOrderItemsService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private DeliveryOrderItemsMapper deliveryOrderItemsMapper;
    @Autowired
    private DeliveryOrderService deliveryOrderService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<DeliveryOrderItems> mapper() {
        return deliveryOrderItemsMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DeliveryOrderItems saveNew(DeliveryOrderItems deliveryOrderItems) {
        deliveryOrderItemsMapper.insert(this.buildDefaultValue(deliveryOrderItems));
        return deliveryOrderItems;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<DeliveryOrderItems> deliveryOrderItemsList) {
        if (CollUtil.isNotEmpty(deliveryOrderItemsList)) {
            deliveryOrderItemsList.forEach(this::buildDefaultValue);
            deliveryOrderItemsMapper.insertList(deliveryOrderItemsList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<DeliveryOrderItems> deliveryOrderItemsList, int batchSize) {
        if (CollUtil.isEmpty(deliveryOrderItemsList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(deliveryOrderItemsList.size(), start + batchSize);
            List<DeliveryOrderItems> subList = deliveryOrderItemsList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            deliveryOrderItemsMapper.insertList(subList);
            if (end == deliveryOrderItemsList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(DeliveryOrderItems deliveryOrderItems, DeliveryOrderItems originalDeliveryOrderItems) {
        MyModelUtil.fillCommonsForUpdate(deliveryOrderItems, originalDeliveryOrderItems);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<DeliveryOrderItems> uw = this.createUpdateQueryForNullValue(deliveryOrderItems, deliveryOrderItems.getId());
        return deliveryOrderItemsMapper.update(deliveryOrderItems, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return deliveryOrderItemsMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByOrderId(Long orderId) {
        DeliveryOrderItems deletedObject = new DeliveryOrderItems();
        deletedObject.setOrderId(orderId);
        return deliveryOrderItemsMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateBatchByOrderId(Long orderId, List<DeliveryOrderItems> dataList) {
        this.updateBatchOneToManyRelation("orderId", orderId,
                null, null, dataList, this::saveNewBatch);
    }

    @Override
    public List<DeliveryOrderItems> getDeliveryOrderItemsList(DeliveryOrderItems filter, String orderBy) {
        return deliveryOrderItemsMapper.getDeliveryOrderItemsList(filter, orderBy);
    }

    @Override
    public List<DeliveryOrderItems> getDeliveryOrderItemsListWithRelation(DeliveryOrderItems filter, String orderBy) {
        List<DeliveryOrderItems> resultList = deliveryOrderItemsMapper.getDeliveryOrderItemsList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<DeliveryOrderItems> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        return CallResult.ok();
    }

    @Override
    public List<DeliveryOrderItems> getDeliveryOrderItemsVoList(Set<Long> orderIds) {
        LambdaQueryWrapper<DeliveryOrderItems> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(DeliveryOrderItems::getOrderId, orderIds);
        List<DeliveryOrderItems> deliveryOrderItems = deliveryOrderItemsMapper.selectList(queryWrapper);
        return deliveryOrderItems;
    }

    @Override
    public CallResult verifyRelatedData(DeliveryOrderItems deliveryOrderItems, DeliveryOrderItems originalDeliveryOrderItems) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是一对多的验证
        if (this.needToVerify(deliveryOrderItems, originalDeliveryOrderItems, DeliveryOrderItems::getOrderId)
                && !deliveryOrderService.existId(deliveryOrderItems.getOrderId())) {
            return CallResult.error(String.format(errorMessageFormat, "订单id"));
        }
        return CallResult.ok();
    }

    private DeliveryOrderItems buildDefaultValue(DeliveryOrderItems deliveryOrderItems) {
        if (deliveryOrderItems.getId() == null) {
            deliveryOrderItems.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(deliveryOrderItems);
        return deliveryOrderItems;
    }
}
