package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 劳务图集数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("jobFileService")
public class JobFileServiceImpl extends BaseService<JobFile, Long> implements JobFileService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private JobFileMapper jobFileMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<JobFile> mapper() {
        return jobFileMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public JobFile saveNew(JobFile jobFile) {
        jobFileMapper.insert(this.buildDefaultValue(jobFile));
        return jobFile;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<JobFile> jobFileList) {
        if (CollUtil.isNotEmpty(jobFileList)) {
            jobFileList.forEach(this::buildDefaultValue);
            jobFileMapper.insertList(jobFileList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(JobFile jobFile, JobFile originalJobFile) {
        MyModelUtil.fillCommonsForUpdate(jobFile, originalJobFile);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<JobFile> uw = this.createUpdateQueryForNullValue(jobFile, jobFile.getId());
        return jobFileMapper.update(jobFile, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return jobFileMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByAssociationTableId(Long associationTableId) {
        JobFile deletedObject = new JobFile();
        deletedObject.setAssociationTableId(associationTableId);
        return jobFileMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Override
    public List<JobFile> getJobFileList(JobFile filter, String orderBy) {
        return jobFileMapper.getJobFileList(filter, orderBy);
    }

    @Override
    public List<JobFile> getJobFileListWithRelation(JobFile filter, String orderBy) {
        List<JobFile> resultList = jobFileMapper.getJobFileList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private JobFile buildDefaultValue(JobFile jobFile) {
        if (jobFile.getId() == null) {
            jobFile.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(jobFile);
        jobFile.setDataState(GlobalDeletedFlag.NORMAL);
        return jobFile;
    }
}
