package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 旅游项目游记管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourTourismProjectTravelNotesService")
public class TourTourismProjectTravelNotesServiceImpl extends BaseService<TourTourismProjectTravelNotes, Long> implements TourTourismProjectTravelNotesService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourTourismProjectTravelNotesMapper tourTourismProjectTravelNotesMapper;
    @Autowired
    private DirectoryInfoService directoryInfoService;
    @Autowired
    private TourTourismTravelNotesFileService tourTourismTravelNotesFileService;
    @Autowired
    private TourTourismTravelNotesContentService tourTourismTravelNotesContentService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourTourismProjectTravelNotes> mapper() {
        return tourTourismProjectTravelNotesMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourTourismProjectTravelNotes saveNew(TourTourismProjectTravelNotes tourTourismProjectTravelNotes) {
        tourTourismProjectTravelNotesMapper.insert(this.buildDefaultValue(tourTourismProjectTravelNotes));
        return tourTourismProjectTravelNotes;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList) {
        if (CollUtil.isNotEmpty(tourTourismProjectTravelNotesList)) {
            tourTourismProjectTravelNotesList.forEach(this::buildDefaultValue);
            tourTourismProjectTravelNotesMapper.insertList(tourTourismProjectTravelNotesList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourTourismProjectTravelNotes saveNewWithRelation(TourTourismProjectTravelNotes tourTourismProjectTravelNotes, JSONObject relationData) {
        this.saveNew(tourTourismProjectTravelNotes);
        this.saveOrUpdateOneToOneRelationData(tourTourismProjectTravelNotes, relationData);
        return tourTourismProjectTravelNotes;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourTourismProjectTravelNotes tourTourismProjectTravelNotes, TourTourismProjectTravelNotes originalTourTourismProjectTravelNotes) {
        MyModelUtil.fillCommonsForUpdate(tourTourismProjectTravelNotes, originalTourTourismProjectTravelNotes);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourTourismProjectTravelNotes> uw = this.createUpdateQueryForNullValue(tourTourismProjectTravelNotes, tourTourismProjectTravelNotes.getId());
        return tourTourismProjectTravelNotesMapper.update(tourTourismProjectTravelNotes, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            TourTourismProjectTravelNotes tourTourismProjectTravelNotes, TourTourismProjectTravelNotes originalTourTourismProjectTravelNotes, JSONObject relationData) {
        // tourTourismProjectTravelNotes 为空的时候，无需修改主表数据。
        if (tourTourismProjectTravelNotes != null && !this.update(tourTourismProjectTravelNotes, originalTourTourismProjectTravelNotes)) {
            return false;
        }
        this.saveOrUpdateOneToOneRelationData(originalTourTourismProjectTravelNotes, relationData);
        return true;
    }

    private void saveOrUpdateOneToOneRelationData(TourTourismProjectTravelNotes tourTourismProjectTravelNotes, JSONObject relationData) {
        // 对于一对一新增或更新，如果主键值为空就新增，否则就更新，同时更新updateTime和updateUserId。
        TourTourismTravelNotesFile tourTourismTravelNotesFile = relationData.getObject("tourTourismTravelNotesFile", TourTourismTravelNotesFile.class);
        if (tourTourismTravelNotesFile != null) {
            tourTourismTravelNotesFile.setAssociationTableId(tourTourismProjectTravelNotes.getId());
            tourTourismTravelNotesFileService.saveNewOrUpdate(tourTourismTravelNotesFile,
                    tourTourismTravelNotesFileService::saveNew, tourTourismTravelNotesFileService::update);
        }
        TourTourismTravelNotesContent tourTourismTravelNotesContent = relationData.getObject("tourTourismTravelNotesContent", TourTourismTravelNotesContent.class);
        if (tourTourismTravelNotesContent != null) {
            tourTourismTravelNotesContent.setAssociationId(tourTourismProjectTravelNotes.getId());
            tourTourismTravelNotesContentService.saveNewOrUpdate(tourTourismTravelNotesContent,
                    tourTourismTravelNotesContentService::saveNew, tourTourismTravelNotesContentService::update);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (tourTourismProjectTravelNotesMapper.deleteById(id) == 0) {
            return false;
        }
        tourTourismTravelNotesFileService.removeByAssociationTableId(id);
        tourTourismTravelNotesContentService.removeByAssociationId(id);
        return true;
    }

    @Override
    public List<TourTourismProjectTravelNotes> getTourTourismProjectTravelNotesList(TourTourismProjectTravelNotes filter, String orderBy) {
        return tourTourismProjectTravelNotesMapper.getTourTourismProjectTravelNotesList(filter, orderBy);
    }

    @Override
    public List<TourTourismProjectTravelNotes> getTourTourismProjectTravelNotesListWithRelation(TourTourismProjectTravelNotes filter, String orderBy) {
        List<TourTourismProjectTravelNotes> resultList = tourTourismProjectTravelNotesMapper.getTourTourismProjectTravelNotesList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourTourismProjectTravelNotes tourTourismProjectTravelNotes, TourTourismProjectTravelNotes originalTourTourismProjectTravelNotes) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourTourismProjectTravelNotes, originalTourTourismProjectTravelNotes, TourTourismProjectTravelNotes::getBelongTab)
                && !directoryInfoService.existId(tourTourismProjectTravelNotes.getBelongTab())) {
            return CallResult.error(String.format(errorMessageFormat, "所属分类"));
        }
        return CallResult.ok();
    }

    private TourTourismProjectTravelNotes buildDefaultValue(TourTourismProjectTravelNotes tourTourismProjectTravelNotes) {
        if (tourTourismProjectTravelNotes.getId() == null) {
            tourTourismProjectTravelNotes.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourTourismProjectTravelNotes);
        tourTourismProjectTravelNotes.setDataState(GlobalDeletedFlag.NORMAL);
        return tourTourismProjectTravelNotes;
    }
}
