package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 旅游富文本数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourismContentService")
public class TourismContentServiceImpl extends BaseService<TourismContent, Long> implements TourismContentService {

    @Autowired
    private TourismContentMapper tourismContentMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourismContent> mapper() {
        return tourismContentMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourismContent saveNew(TourismContent tourismContent) {
        tourismContentMapper.insert(this.buildDefaultValue(tourismContent));
        return tourismContent;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourismContent> tourismContentList) {
        if (CollUtil.isNotEmpty(tourismContentList)) {
            tourismContentList.forEach(this::buildDefaultValue);
            tourismContentMapper.insertList(tourismContentList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourismContent tourismContent, TourismContent originalTourismContent) {
        MyModelUtil.fillCommonsForUpdate(tourismContent, originalTourismContent);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourismContent> uw = this.createUpdateQueryForNullValue(tourismContent, tourismContent.getId());
        return tourismContentMapper.update(tourismContent, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourismContentMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByAssociationId(Long associationId) {
        TourismContent deletedObject = new TourismContent();
        deletedObject.setAssociationId(associationId);
        return tourismContentMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Override
    public List<TourismContent> getTourismContentList(TourismContent filter, String orderBy) {
        return tourismContentMapper.getTourismContentList(filter, orderBy);
    }

    @Override
    public List<TourismContent> getTourismContentListWithRelation(TourismContent filter, String orderBy) {
        List<TourismContent> resultList = tourismContentMapper.getTourismContentList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourismContent buildDefaultValue(TourismContent tourismContent) {
        MyModelUtil.fillCommonsForInsert(tourismContent);
        tourismContent.setDataState(GlobalDeletedFlag.NORMAL);
        return tourismContent;
    }
}
