package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 旅游图集数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourismFileService")
public class TourismFileServiceImpl extends BaseService<TourismFile, Long> implements TourismFileService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourismFileMapper tourismFileMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourismFile> mapper() {
        return tourismFileMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourismFile saveNew(TourismFile tourismFile) {
        tourismFileMapper.insert(this.buildDefaultValue(tourismFile));
        return tourismFile;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourismFile> tourismFileList) {
        if (CollUtil.isNotEmpty(tourismFileList)) {
            tourismFileList.forEach(this::buildDefaultValue);
            tourismFileMapper.insertList(tourismFileList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourismFile tourismFile, TourismFile originalTourismFile) {
        MyModelUtil.fillCommonsForUpdate(tourismFile, originalTourismFile);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourismFile> uw = this.createUpdateQueryForNullValue(tourismFile, tourismFile.getId());
        return tourismFileMapper.update(tourismFile, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourismFileMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByAssociationTableId(Long associationTableId) {
        TourismFile deletedObject = new TourismFile();
        deletedObject.setAssociationTableId(associationTableId);
        return tourismFileMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Override
    public List<TourismFile> getTourismFileList(TourismFile filter, String orderBy) {
        return tourismFileMapper.getTourismFileList(filter, orderBy);
    }

    @Override
    public List<TourismFile> getTourismFileListWithRelation(TourismFile filter, String orderBy) {
        List<TourismFile> resultList = tourismFileMapper.getTourismFileList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourismFile buildDefaultValue(TourismFile tourismFile) {
        if (tourismFile.getId() == null) {
            tourismFile.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourismFile);
        tourismFile.setDataState(GlobalDeletedFlag.NORMAL);
        return tourismFile;
    }
}
