package com.tourism.webadmin.back.vo;

import com.tourism.common.core.base.vo.BaseVo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 餐馆美食信息VO视图对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "RestaurantFoodInfoVO视图对象")
@Data
@EqualsAndHashCode(callSuper = true)
public class RestaurantFoodInfoVo extends BaseVo {

    /**
     * 主键id。
     */
    @Schema(description = "主键id")
    private String id;

    /**
     * 饭馆美食种类id。
     */
    @Schema(description = "饭馆美食种类id")
    private String foodTypeId;

    /**
     * 菜品名称。
     */
    @Schema(description = "菜品名称")
    private String name;

    /**
     * 菜品图片。
     */
    @Schema(description = "菜品图片")
    private String url;

    /**
     * 是否启用，0禁用，1启用。
     */
    @Schema(description = "是否启用，0禁用，1启用")
    private Integer enable;

    /**
     * 菜品销量。
     */
    @Schema(description = "菜品销量")
    private Integer sales;

    /**
     * 菜品价格。
     */
    @Schema(description = "菜品价格")
    private BigDecimal price;

    /**
     * 价格单位。
     */
    @Schema(description = "价格单位")
    private String priceUnit;

    /**
     * 用户所在部门Id。
     */
    @Schema(description = "用户所在部门Id")
    private Long restaurantId;

    /**
     * 打包费。
     */
    @Schema(description = "打包费")
    private BigDecimal balingCharge;

    /**
     * foodTypeId 字典关联数据。
     */
    @Schema(description = "foodTypeId 字典关联数据")
    private Map<String, Object> foodTypeIdDictMap;

    /**
     * restaurantId 字典关联数据。
     */
    @Schema(description = "restaurantId 字典关联数据")
    private Map<String, Object> restaurantIdDictMap;

    /**
     * enable 常量字典关联数据。
     */
    @Schema(description = "enable 常量字典关联数据")
    private Map<String, Object> enableDictMap;

    /**
     * urlAfterConvert url转换后的地址。
     */
    @Schema(description = "urlAfterConvert url转换后的地址")
    private List<String> urlAfterConvert;
}
