package com.tourism.webadmin.back.vo;

import com.tourism.common.core.base.vo.BaseVo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 旅游项目游记管理VO视图对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "TourTourismProjectTravelNotesVO视图对象")
@Data
@EqualsAndHashCode(callSuper = true)
public class TourTourismProjectTravelNotesVo extends BaseVo {

    /**
     * 主键id。
     */
    @Schema(description = "主键id")
    private String id;

    /**
     * 项目标题。
     */
    @Schema(description = "项目标题")
    private String projectTitle;

    /**
     * 项目简述。
     */
    @Schema(description = "项目简述")
    private String remarks;

    /**
     * 是否热点，0非热点，1热点。
     */
    @Schema(description = "是否热点，0非热点，1热点")
    private Integer isHotspot;

    /**
     * 标签。
     */
    @Schema(description = "标签")
    private String projectLabel;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序")
    private Integer showOrder;

    /**
     * 是否启用，0禁用，1启用。
     */
    @Schema(description = "是否启用，0禁用，1启用")
    private Integer enable;

    /**
     * 所属分类。
     */
    @Schema(description = "所属分类")
    private String belongTab;

    /**
     * 出发地。
     */
    @Schema(description = "出发地")
    private String startPlace;

    /**
     * 目的地。
     */
    @Schema(description = "目的地")
    private String endPlace;

    /**
     * 总天数。
     */
    @Schema(description = "总天数")
    private String countTimes;

    /**
     * 项目价格。
     */
    @Schema(description = "项目价格")
    private BigDecimal price;

    /**
     * 项目展示图片。
     */
    @Schema(description = "项目展示图片")
    private String tourismUrl;

    /**
     * 价格单位。
     */
    @Schema(description = "价格单位")
    private String priceUnit;

    /**
     * 服务保障。
     */
    @Schema(description = "服务保障")
    private String serviceEnsure;

    /**
     * 联系电话。
     */
    @Schema(description = "联系电话")
    private String contactNumber;

    /**
     * 产品卖点。
     */
    @Schema(description = "产品卖点")
    private String sellingPoint;

    /**
     * 短标题。
     */
    @Schema(description = "短标题")
    private String shortTitle;

    /**
     * 短描述。
     */
    @Schema(description = "短描述")
    private String shortDescription;

    /**
     * 首页热门图片。
     */
    @Schema(description = "首页热门图片")
    private String homeHotPicture;

    /**
     * 联系人描述。
     */
    @Schema(description = "联系人描述")
    private String contactDescription;

    /**
     * 游记banner。
     */
    @Schema(description = "游记banner")
    private String travelNotesBanner;

    /**
     * 点赞量。
     */
    @Schema(description = "点赞量")
    private Integer likeCount;

    /**
     * 浏览量。
     */
    @Schema(description = "浏览量")
    private Integer pageViewCount;

    /**
     * 人物。
     */
    @Schema(description = "人物")
    private String role;

    /**
     * 出发时间。
     */
    @Schema(description = "出发时间")
    private Date departureTime;

    /**
     * 人均费用。
     */
    @Schema(description = "人均费用")
    private String averageCost;

    /**
     * 推荐指数。
     */
    @Schema(description = "推荐指数")
    private Integer recommendationRate;

    /**
     * id 的一对一关联数据对象，数据对应类型为TourTourismTravelNotesFileVo。
     */
    @Schema(description = "id 的一对一关联数据对象，数据对应类型为TourTourismTravelNotesFileVo")
    private Map<String, Object> tourTourismTravelNotesFile;

    /**
     * id 的一对一关联数据对象，数据对应类型为TourTourismTravelNotesContentVo。
     */
    @Schema(description = "id 的一对一关联数据对象，数据对应类型为TourTourismTravelNotesContentVo")
    private Map<String, Object> tourTourismTravelNotesContent;

    /**
     * belongTab 字典关联数据。
     */
    @Schema(description = "belongTab 字典关联数据")
    private Map<String, Object> belongTabDictMap;

    /**
     * isHotspot 常量字典关联数据。
     */
    @Schema(description = "isHotspot 常量字典关联数据")
    private Map<String, Object> isHotspotDictMap;

    /**
     * enable 常量字典关联数据。
     */
    @Schema(description = "enable 常量字典关联数据")
    private Map<String, Object> enableDictMap;

    /**
     * tourismUrl 转换后的图片对象。
     */
    @Schema(description = "tourismUrl转换后的图片对象")
    private List<String> tourismUrlsAfterConvert;

    /**
     * homeHotPicture 转换后的图片对象。
     */
    @Schema(description = "homeHotPicture转换后的图片对象")
    private List<String> homeHotPicturesAfterConvert;

    /**
     * travelNotesBanner 转换后的图片对象。
     */
    @Schema(description = "travelNotesBanner转换后的图片对象")
    private List<String> travelNotesBannerAfterConvert;

    /**
     * 旅游项目(以逗号分割，最多为三个项目)。
     */
    @Schema(description = "旅游项目(以逗号分割，最多为三个项目)")
    private String projectIds;

    /**
     * projectIds 的一对一关联数据对象，数据对应类型为TourismProjectVo。
     */
    @Schema(description = "projectIds 的一对一关联数据对象，数据对应类型为TourismProjectVo")
    private Map<String, Object> tourismProject;
}
